package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;

/** action for classifier browsing */
public class BrowseAction extends Action {
    
    public static final String CLASSIFIER_ARG = "cl";

    /* add the action specific args to the cgi param list
     */
    public void addCGIParams() {
	cgi_.addStaticParam(CLASSIFIER_ARG);
    }

    /** process the request */
    public Element process (Element message) {

	// get the request - assume only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	
	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	return classifierBrowse(page, request);
	
    }
    

    protected Element classifierBrowse(Element page, Element request) {

	// check that the stylesheet is present - cant output a page without one. we may adapt this to use unknownquery stylesheet? - or ask for one from the MR
	String stylesheet = GSFile.stylesheetFile(config_, "classifier.xsl");
	if (stylesheet==null) {
	    System.err.println("BrowseAction Error: classifier stylesheet not found!");
	    return null;
	}
	Document style_doc = converter_.getDOM(new File(stylesheet));

	// extract the params from the cgi-request, and check that we have a coll specified
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);

	String service_name = (String)params.get(GSCGI.SERVICE_ARG);
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	if (collection == null || collection.equals("")) {
	    System.err.println("BrowseAction Error:classifierbrowse, need to specify a collection!");
	    return null;
	   
	}

	// get the service info from the MR - this will probably need to be cached somehow later on. 

	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	mr_info_request.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));

	String to = collection;
	to = GSPath.appendLink(to, service_name);
	mr_info_request.setAttribute(GSXML.TO_ATT, to);

	Element mr_info_response = (Element) mr_.process(mr_info_message);

	String path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);

	Element description = (Element)doc_.importNode(GSXML.getNodeByPath(mr_info_response, path), true);
	
	// the first part of the data for the page is the cgi-params
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	page.appendChild(description);
	
	// if there is a cl field in the cgi params, get the classifier info
	String node = (String)params.get(CLASSIFIER_ARG);
	
	if (node!=null && !node.equals("")) {

	    // get the info for the selected node
	    Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_query_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_query_message.appendChild(mr_query_request);
	    
	    mr_query_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	    mr_query_request.setAttribute(GSXML.TO_ATT, to);
	    mr_query_request.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	    
	    
	    // paramList - empty for now
	    Element paramList = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	    mr_query_request.appendChild(paramList);
	    
	    Element document_list = doc_.createElement(GSXML.CLASS_NODE_ELEM+GSXML.LIST_MODIFIER);
	    Element document = doc_.createElement(GSXML.CLASS_NODE_ELEM);
	    document.setAttribute(GSXML.CLASS_NODE_ID_ATT, node);
	    document_list.appendChild(document);
	    mr_query_request.appendChild(document_list);
	    
	    Element mr_query_response = (Element)mr_.process(mr_query_message);
	    
	    Element response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	    Node new_style = GSXML.getChildByTagName(response, GSXML.STYLESHEET_ELEM);
	    if (new_style !=null) {
		GSXSLT.mergeStylesheets(style_doc, (Element)new_style);
		response.removeChild(new_style);
	    }
	    // add the response to the page data
	    page.appendChild(doc_.importNode(response, true));

	}
	
	GSXSLT.absoluteIncludePaths(style_doc, config_);

	//System.out.println("page=");
	//System.out.println(converter_.getString(page));
	return (Element)transformer_.transform(style_doc, page);
    }

    protected Element unknownBrowse(Element page, Element request, String browse_type) {
	System.err.println("BrowseAction Error: unknown browse subtype: "+browse_type);
	return null;
    }
}


