/*
 *    GS2Browse.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;

import org.greenstone.gdbm.*;
import org.greenstone.gsdl3.util.*;

import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 
/**
 * A ServiceRack class for GSDL2-style Classifiers
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3866 $
 */
public class GS2Browse 
    extends ServiceRack {

    // the services on offer
    private static final String CLASSIFIER_SERVICE = "ClassifierBrowse";

    /** gdbm wrapper */
    private GDBMWrapper gdbm_src_=null;

    public GS2Browse() {
	gdbm_src_ = new GDBMWrapper();

    }
 
    /** creates a display element containing all the text strings needed to display the service page, in the language specified */
    protected Element createServiceDisplay(String service, String lang) {
	Element display = doc_.createElement(GSXML.DISPLAY_ELEM);
	display.appendChild(GSXML.createTextElement(doc_, GSXML.DISPLAY_NAME_ELEM,  getTextString(service+".name", lang)));
	//display.appendChild(GSXML.createTextElement(doc_,  GSXML.DISPLAY_SUBMIT_ELEM, getTextString(service+".submit", lang)));
	
	// need to add in the classifier name info
	// add <classifier name="CL1">text name</classifier> to the 
	// display node
	NodeList classifiers = config_info_.getElementsByTagName(GSXML.CLASSIFIER_ELEM);
	for (int i=0; i<classifiers.getLength(); i++) {
	    Element cl = (Element)classifiers.item(i);
	    Element disp = (Element)GSXML.getChildByTagName(cl, GSXML.DISPLAY_ELEM);
	    String text = null;
	    if (disp !=null) {
		text = GSXML.getDisplayText(disp, 
					    GSXML.DISPLAY_NAME_ELEM, 
					    lang, "en");
	    }
	    if (text == null || text.equals("")) {
		// no display element was specified, use the metadata name
		// for now this looks in the class properties file
		// this needs to use a general metadata thing instead
		text = getTextString(cl.getAttribute(GSXML.CLASSIFIER_CONTENT_ATT), lang);
	    }
	    
	    Element cl_elem = doc_.createElement(GSXML.CLASSIFIER_ELEM);
	    cl_elem.setAttribute(GSXML.NAME_ATT, cl.getAttribute(GSXML.NAME_ATT));
	    Element cl_name = GSXML.createTextElement(doc_, GSXML.DISPLAY_NAME_ELEM, text);
	    cl_elem.appendChild(cl_name);
		
	    display.appendChild(cl_elem);
	    
	}
	
	return display;
	
    }

    /** processes the request */
    protected Element processClassifierBrowse(Element request) {

	// the response element
	Element result = doc_.createElement(GSXML.RESPONSE_ELEM);
	String from = GSPath.appendLink(cluster_name_, CLASSIFIER_SERVICE);
	result.setAttribute(GSXML.FROM_ATT, from);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);

	// get the classifier name out of request content - assume only one for now
	//String path = GSXML.CONTENT_ELEM;
	String path = GSPath.appendLink(GSXML.CLASS_NODE_ELEM+GSXML.LIST_MODIFIER, GSXML.CLASS_NODE_ELEM);
	//path = GSPath.appendLink(path, GSXML.DOCUMENT_ELEM);

	Element elem = (Element)GSXML.getNodeByPath(request, path);
	String elem_name = elem.getAttribute(GSXML.CLASS_NODE_ID_ATT);

	if (OID.needsTranslating(elem_name)) {
	    elem_name = gdbm_src_.translateOID(elem_name);
	}
	// get the info from gdbm db
	DBInfo info = gdbm_src_.getInfo(elem_name);

	// build up the XML
	Element classifier = doc_.createElement(GSXML.CLASSIFIER_ELEM);
	
	String classifier_name = OID.getTop(elem_name);
	classifier.setAttribute(GSXML.NAME_ATT, classifier_name);
	result.appendChild(classifier);

	// see if there is a stylesheet associated with this classifier
	Element cl_list = (Element)GSXML.getChildByTagName(config_info_, GSXML.CLASSIFIER_ELEM+GSXML.LIST_MODIFIER);
	Element cl_info = GSXML.getNamedElement(cl_list,
						GSXML.CLASSIFIER_ELEM,
						GSXML.NAME_ATT,
						classifier_name);
	if (cl_info !=null) {
	    
	    // find the format child
	    Element cl_format = (Element)GSXML.getChildByTagName(cl_info, GSXML.FORMAT_ELEM);
	    // add it to the result if the result isnt null
	    if (cl_format!=null) {
		result.appendChild(doc_.importNode(cl_format, true));
	    }
	}
		
	DBInfo top_info;

	if (elem_name.equals(classifier_name)) { 
	    top_info = info;
	} else {
	    top_info = gdbm_src_.getInfo(classifier_name);
	}

	// get the classifier attributes
	String type = top_info.getInfo("childtype");
	if (type.equals("VList")) {
	    
	    classifier.setAttribute("type", "vertical");
	    classifier.setAttribute("interleave", "true");
	    classifier.setAttribute("link", "icon");
	}
	else if (type.equals("HList")){
	    classifier.setAttribute("type", "horizontal");
	    classifier.setAttribute("interleave", "false");
	    classifier.setAttribute("link", "name");
	} else {
	    System.out.println("unknown type!!! check this");
	    classifier.setAttribute("type", "vertical");
	    classifier.setAttribute("interleave", "true");
	    classifier.setAttribute("link", "icon");
	}

	Element this_node;
	if (elem_name.equals(classifier_name)) {
	    this_node = classifier;
	} else { // have to build up the parents 
	    // the current node
	    this_node = doc_.createElement("node");
	    this_node.setAttribute("name", elem_name);
	    type = info.getInfo("childtype");
	    if (type.equals("VList")) {
		this_node.setAttribute("type", "vertical");
	    } else {
		this_node.setAttribute("type", "horzizontal");
	    }
	    String title = info.getInfo("Title");
	    Element meta_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	    GSXML.addMetadata(doc_, meta_list, "Title", title);
	    this_node.appendChild(meta_list);

	    // the parents
	    String middle_name = OID.getParent(elem_name);
	    Element middle_node = this_node;
	    while (!middle_name.equals(classifier_name)) {
		// add in a new node
		Element temp_node = doc_.createElement("node");
		temp_node.setAttribute("name", middle_name);
		
		DBInfo i = gdbm_src_.getInfo(middle_name);
		type = i.getInfo("childtype");
		if (type.equals("VList")) {
		    temp_node.setAttribute("type", "vertical");
		} else {
		    temp_node.setAttribute("type", "horzizontal");
		}
		title = i.getInfo("Title");
		meta_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
		GSXML.addMetadata(doc_, meta_list, "Title", title);
		temp_node.appendChild(meta_list);

		// link into the tree
		temp_node.appendChild(middle_node);
		middle_node = temp_node;
		middle_name = OID.getParent(middle_name);
	    }
	    classifier.appendChild(middle_node);
	}
	// get the child nodes
	String contains = info.getInfo("contains");
	// replace the " with parent name
	contains = OID.translateParent(contains, elem_name);
	String [] children = contains.split(";");
	for (int i=0; i<children.length; i++) {
	    String child = children[i];
	    DBInfo child_info = gdbm_src_.getInfo(child);
	    String title = child_info.getInfo("Title");
	    Element node;
	    // should have a type here, not rely on HASH ids
	    if (child.startsWith("HASH")) {
		// a document
		node = doc_.createElement(GSXML.DOCUMENT_ELEM);
	    } else { // a node in the classification
		node = doc_.createElement("node");
		
	    }
	    
	    node.setAttribute(GSXML.NAME_ATT, child);
	    Element meta_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	    GSXML.addMetadata(doc_, meta_list, "Title", title);
	    String keyword = child_info.getInfo("Keyword");
	    String source = child_info.getInfo("Source");
	    GSXML.addMetadata(doc_, meta_list, "Keyword", keyword);
	    GSXML.addMetadata(doc_, meta_list, "Source", source);
	    node.appendChild(meta_list);
	    this_node.appendChild(node);
	    
	}
	
	System.out.println("result=");
	System.out.println(converter_.getString(result));
	return result;
	
    }
    

    /** configure this service */
    public boolean configure(Element info) {

	System.out.println("configuring GS2Browse");
	System.out.println("config info=");
	System.out.println(converter_.getString(info));
	Element e = null;
	// these entries should reflect the build config file - some services may not be available depending on how the colleciton was built.
	// set up short_service_info_ - for now just has name and type
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BROWSE);
	e.setAttribute(GSXML.NAME_ATT, CLASSIFIER_SERVICE);
	short_service_info_.appendChild(e);

	// set up config info - just a cache of whats in the config file
	config_info_ = info;

	// set up service_info_map_ - for now, just has the same elements as above
	// should have full details about each service incl params lists etc.
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BROWSE);
	e.setAttribute(GSXML.NAME_ATT, CLASSIFIER_SERVICE);
	Element cl_list = doc_.createElement(GSXML.CLASSIFIER_ELEM+GSXML.LIST_MODIFIER);
	e.appendChild(cl_list);

	// add in to the description a simplified list of classifiers
	NodeList classifiers = config_info_.getElementsByTagName(GSXML.CLASSIFIER_ELEM);
	
	for(int i=0; i<classifiers.getLength(); i++) {
	    Element cl = (Element)classifiers.item(i);
	    Element new_cl = (Element)doc_.importNode(cl, false); // just import this node, not the children
	    cl_list.appendChild(new_cl);
	    
	}
	    
	service_info_map_.put(CLASSIFIER_SERVICE, e);
	
	if (gdbm_src_.openDatabase(GSFile.GDBMDatabaseFile(site_home_, cluster_name_), GDBMWrapper.READER)) {
	    return true;
	}
	else {
	    System.err.println("couldn't open gdbm database!");
	    return false;
	}
	
    }


}  
 

